import bpy
import bmesh
from .utils import get_addon_prefs
from. mesh_utils import calculate_scale, AddModCylinder
from .add_menu import draw_items
from .addon_preferences import CONST_ITEMS

def draw_add_none_destructive_primitive(self, context):
    draw_items(self, context)
    layout = self.layout    
    layout.separator()
    
class AddNoneDestructivePrimitive(bpy.types.Operator):
    bl_idname = "mesh.add_none_destructive_primitive"
    bl_label = "Add None-Destructive Primitive"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Adds a none destructive primitive"

    prefs = get_addon_prefs()

    type: bpy.props.StringProperty() # type: ignore
    auto_scale: bpy.props.BoolProperty(default=True) # type: ignore
    scale: bpy.props.FloatProperty(name="Scale", default=1.0, min=0.1, max=10.0, unit='LENGTH') # type: ignore
    scale_auto_scale: bpy.props.FloatProperty(name="Scale", default=6.0, min=0.1, max=16.0) # type: ignore

    def draw(self, context):
        layout = self.layout
        text = self.type
        icon = next((item[1] for item in CONST_ITEMS if item[0] == self.type), 'FILE_3D')
        layout.label(text=text, icon=icon)
        row = layout.row()
        row.alignment = 'LEFT'
        row.prop(self, "auto_scale")
        if not self.auto_scale:
            row.prop(self, "scale")
        else:
            row.prop(self, "scale_auto_scale")

    def execute(self, context):
        prefs = get_addon_prefs()
        file_path = __file__.replace('add_none_destructive_primitive.py', 'Primitives.blend')
        name = self.type

        bpy.ops.object.select_all(action='DESELECT') # do we whant it to deselect?

        if self.type == 'Mod Cylinder':
            AddModCylinder(context, 16,  self.auto_scale, 6.0, self.scale)   
            bpy.app.timers.register(switch_to_modifier_context, first_interval=0.001) # workaround to switch to modifier context, otherwise error if it does not exist already
            return {'FINISHED'}

        if name not in bpy.data.node_groups:
            with bpy.data.libraries.load(file_path, link=False) as (data_from, data_to):
                data_to.node_groups = [name for name in data_from.node_groups if name == name]

        new_mesh = bpy.data.meshes.new(name)
        bm = bmesh.new()
        bm.from_mesh(new_mesh)
        bm.verts.new((0,0,0)) # add single vert so we get a material index
        bm.to_mesh(new_mesh)
        bm.free()

        new_obj = bpy.data.objects.new(name, new_mesh)
        context.collection.objects.link(new_obj)
        # move to cursor
        new_obj.location = context.scene.cursor.location

        # add geometry node modifier to object
        mod = new_obj.modifiers.new(name=name, type='NODES')
        new_obj.modifiers[name].node_group = bpy.data.node_groups[name]

        # set active object + select
        new_obj.select_set(True)
        context.view_layer.objects.active = new_obj
        
        # add object properties
        new_obj['none_destructive_primitive'] = name

        # scale object
        scale = calculate_scale(context, self.scale, self.auto_scale, self.scale_auto_scale, custom_scale_factor=1.0)
        
        # set scale to geometry node modifier based on socket name
        double_scale = []
        full_scale = ["Depth", "Size", "Size X", "Size Y","Height"]
        half_scale = ["Radius", "Radius Bottom", "Start Radius", "End Radius"]
        one_fourth_scale = ["Minor Radius"]
        if name == 'Torus':
            full_scale = ["Major Radius"]
        if name == 'Quad Sphere':
            full_scale = []
            half_scale = ["Size"]
        if name == 'Capsule':
            double_scale = ["Height"]
            half_scale = []
            one_fourth_scale = ["Radius"]
   
        node_group = mod.node_group
        input_node = next((node for node in node_group.nodes if node.type == 'GROUP_INPUT'),None)
        
        for node_output in input_node.outputs[:-1]:
            if not node_output.type == 'GEOMETRY':
                socket_id = node_output.identifier
                socket_type = node_output.type

                if socket_type == 'VECTOR':
                    scale = scale, scale, scale

                if node_output.name in double_scale:
                    mod[socket_id] = scale * 2
                if node_output.name in full_scale:
                    mod[socket_id] = scale
                elif node_output.name in half_scale:
                    mod[socket_id] = scale / 2
                elif node_output.name in one_fourth_scale:
                    mod[socket_id] = scale / 4

        # loop troght all nods and mute gizmo nodes if prefs.show_gizmo is False
        if not prefs.show_gizmo:
            for node in node_group.nodes:
                if node.type == 'GIZMO_DIAL' or node.type == 'GIZMO_LINEAR':
                    if not node.mute:
                        node.mute = True
                
        # register timer to switch to modifier context¨
        if prefs.auto_swith_to_modifier_tab:
            bpy.app.timers.register(switch_to_modifier_context, first_interval=0.005) # workaround to switch to modifier context, otherwise error if it does not exist already
        return {'FINISHED'}
    
def switch_to_modifier_context():
    for area in bpy.context.screen.areas:
        if area.type == 'PROPERTIES':
            area.spaces[0].context = 'MODIFIER'